@@ -7,7 +7,7 @@ use floating_pane_grid::FloatingPaneGrid;
 use crate::{
     os_input_output::ServerOsApi,
     output::{FloatingPanesStack, Output},
-    panes::PaneId,
+    panes::{ActivePanes, PaneId},
     ui::pane_contents_and_ui::PaneContentsAndUi,
     ClientId,
 };
@@ -17,6 +17,7 @@ use std::rc::Rc;
 use std::time::Instant;
 use zellij_utils::{
     data::{ModeInfo, Style},
+    errors::prelude::*,
     input::command::RunCommand,
     pane_size::{Offset, PaneGeom, Size, Viewport},
 };
@@ -30,7 +31,9 @@ macro_rules! resize_pty {
                 *pid,
                 $pane.get_content_columns() as u16,
                 $pane.get_content_rows() as u16,
-            );
+            )
+        } else {
+            Ok(())
         }
     };
 }
@@ -47,7 +50,7 @@ pub struct FloatingPanes {
     session_is_mirrored: bool,
     desired_pane_positions: HashMap<PaneId, PaneGeom>, // this represents the positions of panes the user moved with intention, rather than by resizing the terminal window
     z_indices: Vec<PaneId>,
-    active_panes: HashMap<ClientId, PaneId>,
+    active_panes: ActivePanes,
     show_panes: bool,
     pane_being_moved_with_mouse: Option<(PaneId, Position)>,
 }
@@ -64,6 +67,7 @@ impl FloatingPanes {
         session_is_mirrored: bool,
         default_mode_info: ModeInfo,
         style: Style,
+        os_input: Box<dyn ServerOsApi>,
     ) -> Self {
         FloatingPanes {
             panes: BTreeMap::new(),
@@ -78,7 +82,7 @@ impl FloatingPanes {
             desired_pane_positions: HashMap::new(),
             z_indices: vec![],
             show_panes: false,
-            active_panes: HashMap::new(),
+            active_panes: ActivePanes::new(&os_input),
             pane_being_moved_with_mouse: None,
         }
     }
@@ -155,11 +159,12 @@ impl FloatingPanes {
         &mut self,
         pane_id: PaneId,
         exit_status: Option<i32>,
+        is_first_run: bool,
         run_command: RunCommand,
     ) {
         self.panes
             .get_mut(&pane_id)
-            .map(|p| p.hold(exit_status, run_command));
+            .map(|p| p.hold(exit_status, is_first_run, run_command));
     }
     pub fn get(&self, pane_id: &PaneId) -> Option<&Box<dyn Pane>> {
         self.panes.get(pane_id)
@@ -191,6 +196,11 @@ impl FloatingPanes {
     }
     pub fn toggle_show_panes(&mut self, should_show_floating_panes: bool) {
         self.show_panes = should_show_floating_panes;
+        if should_show_floating_panes {
+            self.active_panes.focus_all_panes(&mut self.panes);
+        } else {
+            self.active_panes.unfocus_all_panes(&mut self.panes);
+        }
     }
     pub fn active_panes_contain(&self, client_id: &ClientId) -> bool {
         self.active_panes.contains_key(client_id)
@@ -231,10 +241,11 @@ impl FloatingPanes {
             } else {
                 pane.set_content_offset(Offset::default());
             }
-            resize_pty!(pane, os_api);
+            resize_pty!(pane, os_api).unwrap();
         }
     }
-    pub fn render(&mut self, output: &mut Output) {
+    pub fn render(&mut self, output: &mut Output) -> Result<()> {
+        let err_context = || "failed to render output";
         let connected_clients: Vec<ClientId> =
             { self.connected_clients.borrow().iter().copied().collect() };
         let mut floating_panes: Vec<_> = self.panes.iter_mut().collect();
@@ -242,12 +253,20 @@ impl FloatingPanes {
             self.z_indices
                 .iter()
                 .position(|id| id == *a_id)
-                .unwrap()
-                .cmp(&self.z_indices.iter().position(|id| id == *b_id).unwrap())
+                .with_context(err_context)
+                .fatal()
+                .cmp(
+                    &self
+                        .z_indices
+                        .iter()
+                        .position(|id| id == *b_id)
+                        .with_context(err_context)
+                        .fatal(),
+                )
         });
 
         for (z_index, (kind, pane)) in floating_panes.iter_mut().enumerate() {
-            let mut active_panes = self.active_panes.clone();
+            let mut active_panes = self.active_panes.clone_active_panes();
             let multiple_users_exist_in_session =
                 { self.connected_clients_in_app.borrow().len() > 1 };
             active_panes.retain(|c_id, _| self.connected_clients.borrow().contains(c_id));
@@ -266,23 +285,27 @@ impl FloatingPanes {
                     .get(client_id)
                     .unwrap_or(&self.default_mode_info)
                     .mode;
-                pane_contents_and_ui.render_pane_frame(
-                    *client_id,
-                    client_mode,
-                    self.session_is_mirrored,
-                );
+                pane_contents_and_ui
+                    .render_pane_frame(*client_id, client_mode, self.session_is_mirrored)
+                    .with_context(err_context)?;
                 if let PaneId::Plugin(..) = kind {
-                    pane_contents_and_ui.render_pane_contents_for_client(*client_id);
+                    pane_contents_and_ui
+                        .render_pane_contents_for_client(*client_id)
+                        .with_context(err_context)?;
                 }
                 // this is done for panes that don't have their own cursor (eg. panes of
                 // another user)
-                pane_contents_and_ui.render_fake_cursor_if_needed(*client_id);
+                pane_contents_and_ui
+                    .render_fake_cursor_if_needed(*client_id)
+                    .with_context(err_context)?;
             }
             if let PaneId::Terminal(..) = kind {
                 pane_contents_and_ui
-                    .render_pane_contents_to_multiple_clients(connected_clients.iter().copied());
+                    .render_pane_contents_to_multiple_clients(connected_clients.iter().copied())
+                    .with_context(err_context)?;
             }
         }
+        Ok(())
     }
     pub fn resize(&mut self, new_screen_size: Size) {
         let display_area = *self.display_area.borrow();
@@ -298,7 +321,7 @@ impl FloatingPanes {
     }
     pub fn resize_pty_all_panes(&mut self, os_api: &mut Box<dyn ServerOsApi>) {
         for pane in self.panes.values_mut() {
-            resize_pty!(pane, os_api);
+            resize_pty!(pane, os_api).unwrap();
         }
     }
     pub fn resize_active_pane_left(
@@ -318,7 +341,7 @@ impl FloatingPanes {
             );
             floating_pane_grid.resize_pane_left(active_floating_pane_id);
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api);
+                resize_pty!(pane, os_api).unwrap();
             }
             self.set_force_render();
             return true;
@@ -342,7 +365,7 @@ impl FloatingPanes {
             );
             floating_pane_grid.resize_pane_right(active_floating_pane_id);
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api);
+                resize_pty!(pane, os_api).unwrap();
             }
             self.set_force_render();
             return true;
@@ -366,7 +389,7 @@ impl FloatingPanes {
             );
             floating_pane_grid.resize_pane_down(active_floating_pane_id);
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api);
+                resize_pty!(pane, os_api).unwrap();
             }
             self.set_force_render();
             return true;
@@ -390,7 +413,7 @@ impl FloatingPanes {
             );
             floating_pane_grid.resize_pane_up(active_floating_pane_id);
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api);
+                resize_pty!(pane, os_api).unwrap();
             }
             self.set_force_render();
             return true;
@@ -414,7 +437,7 @@ impl FloatingPanes {
             );
             floating_pane_grid.resize_increase(active_floating_pane_id);
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api);
+                resize_pty!(pane, os_api).unwrap();
             }
             self.set_force_render();
             return true;
@@ -438,7 +461,7 @@ impl FloatingPanes {
             );
             floating_pane_grid.resize_decrease(active_floating_pane_id);
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api);
+                resize_pty!(pane, os_api).unwrap();
             }
             self.set_force_render();
             return true;
@@ -515,7 +538,7 @@ impl FloatingPanes {
             },
             None => {
                 // TODO: can this happen?
-                self.active_panes.clear();
+                self.active_panes.clear(&mut self.panes);
                 self.z_indices.clear();
             },
         }
@@ -586,7 +609,7 @@ impl FloatingPanes {
             },
             None => {
                 // TODO: can this happen?
-                self.active_panes.clear();
+                self.active_panes.clear(&mut self.panes);
                 self.z_indices.clear();
             },
         }
@@ -656,7 +679,7 @@ impl FloatingPanes {
             },
             None => {
                 // TODO: can this happen?
-                self.active_panes.clear();
+                self.active_panes.clear(&mut self.panes);
                 self.z_indices.clear();
             },
         }
@@ -726,7 +749,7 @@ impl FloatingPanes {
             },
             None => {
                 // TODO: can this happen?
-                self.active_panes.clear();
+                self.active_panes.clear(&mut self.panes);
                 self.z_indices.clear();
             },
         }
@@ -799,7 +822,8 @@ impl FloatingPanes {
             if active_pane_id == pane_id {
                 match next_active_pane {
                     Some(next_active_pane) => {
-                        self.active_panes.insert(client_id, next_active_pane);
+                        self.active_panes
+                            .insert(client_id, next_active_pane, &mut self.panes);
                         self.focus_pane(next_active_pane, client_id);
                     },
                     None => {
@@ -813,7 +837,8 @@ impl FloatingPanes {
         let connected_clients: Vec<ClientId> =
             self.connected_clients.borrow().iter().copied().collect();
         for client_id in connected_clients {
-            self.active_panes.insert(client_id, pane_id);
+            self.active_panes
+                .insert(client_id, pane_id, &mut self.panes);
         }
         self.z_indices.retain(|p_id| *p_id != pane_id);
         self.z_indices.push(pane_id);
@@ -821,12 +846,13 @@ impl FloatingPanes {
         self.set_force_render();
     }
     pub fn focus_pane(&mut self, pane_id: PaneId, client_id: ClientId) {
-        self.active_panes.insert(client_id, pane_id);
+        self.active_panes
+            .insert(client_id, pane_id, &mut self.panes);
         self.focus_pane_for_all_clients(pane_id);
     }
     pub fn defocus_pane(&mut self, pane_id: PaneId, client_id: ClientId) {
         self.z_indices.retain(|p_id| *p_id != pane_id);
-        self.active_panes.remove(&client_id);
+        self.active_panes.remove(&client_id, &mut self.panes);
         self.set_force_render();
     }
     pub fn get_pane(&self, pane_id: PaneId) -> Option<&Box<dyn Pane>> {
@@ -943,8 +969,9 @@ impl FloatingPanes {
             .map(|(cid, _pid)| *cid)
             .collect();
         for client_id in clients_in_pane {
-            self.active_panes.remove(&client_id);
-            self.active_panes.insert(client_id, to_pane_id);
+            self.active_panes.remove(&client_id, &mut self.panes);
+            self.active_panes
+                .insert(client_id, to_pane_id, &mut self.panes);
         }
     }
 }
