@@ -4,7 +4,7 @@ use crate::screen::CopyOptions;
 use crate::Arc;
 use crate::Mutex;
 use crate::{
-    os_input_output::{AsyncReader, Pid, ServerOsApi, SpawnTerminalError},
+    os_input_output::{AsyncReader, Pid, ServerOsApi},
     panes::PaneId,
     thread_bus::ThreadSenders,
     ClientId,
@@ -12,7 +12,7 @@ use crate::{
 use std::path::PathBuf;
 use zellij_utils::channels::Receiver;
 use zellij_utils::envs::set_session_name;
-use zellij_utils::errors::ErrorContext;
+use zellij_utils::errors::{prelude::*, ErrorContext};
 use zellij_utils::input::layout::{Layout, PaneLayout};
 use zellij_utils::ipc::IpcReceiverWithContext;
 use zellij_utils::pane_size::{Size, SizeInPixels};
@@ -22,12 +22,10 @@ use crate::pty_writer::PtyWriteInstruction;
 use zellij_utils::channels::{self, ChannelWithContext, SenderWithContext};
 
 use std::cell::RefCell;
-use std::collections::{HashMap, HashSet};
+use std::collections::{BTreeMap, HashMap, HashSet};
 use std::os::unix::io::RawFd;
 use std::rc::Rc;
 
-use zellij_utils::nix;
-
 use zellij_utils::{
     data::{InputMode, ModeInfo, Palette, Style},
     input::command::{RunCommand, TerminalAction},
@@ -35,59 +33,63 @@ use zellij_utils::{
     ipc::{ClientToServerMsg, ServerToClientMsg},
 };
 
-#[derive(Clone)]
+#[derive(Clone, Default)]
 struct FakeInputOutput {
     file_dumps: Arc<Mutex<HashMap<String, String>>>,
+    pub tty_stdin_bytes: Arc<Mutex<BTreeMap<u32, Vec<u8>>>>,
 }
 
 impl ServerOsApi for FakeInputOutput {
-    fn set_terminal_size_using_terminal_id(&self, _id: u32, _cols: u16, _rows: u16) {
+    fn set_terminal_size_using_terminal_id(&self, _id: u32, _cols: u16, _rows: u16) -> Result<()> {
         // noop
+        Ok(())
     }
     fn spawn_terminal(
         &self,
         _file_to_open: TerminalAction,
         _quit_db: Box<dyn Fn(PaneId, Option<i32>, RunCommand) + Send>,
         _default_editor: Option<PathBuf>,
-    ) -> Result<(u32, RawFd, RawFd), SpawnTerminalError> {
+    ) -> Result<(u32, RawFd, RawFd)> {
         unimplemented!()
     }
-    fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize, nix::Error> {
+    fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize> {
         unimplemented!()
     }
     fn async_file_reader(&self, _fd: RawFd) -> Box<dyn AsyncReader> {
         unimplemented!()
     }
-    fn write_to_tty_stdin(&self, _id: u32, _buf: &[u8]) -> Result<usize, nix::Error> {
+    fn write_to_tty_stdin(&self, id: u32, buf: &[u8]) -> Result<usize> {
+        self.tty_stdin_bytes
+            .lock()
+            .unwrap()
+            .entry(id)
+            .or_insert_with(|| vec![])
+            .extend_from_slice(buf);
+        Ok(buf.len())
+    }
+    fn tcdrain(&self, _id: u32) -> Result<()> {
         unimplemented!()
     }
-    fn tcdrain(&self, _id: u32) -> Result<(), nix::Error> {
+    fn kill(&self, _pid: Pid) -> Result<()> {
         unimplemented!()
     }
-    fn kill(&self, _pid: Pid) -> Result<(), nix::Error> {
-        unimplemented!()
-    }
-    fn force_kill(&self, _pid: Pid) -> Result<(), nix::Error> {
+    fn force_kill(&self, _pid: Pid) -> Result<()> {
         unimplemented!()
     }
     fn box_clone(&self) -> Box<dyn ServerOsApi> {
         Box::new((*self).clone())
     }
-    fn send_to_client(
-        &self,
-        _client_id: ClientId,
-        _msg: ServerToClientMsg,
-    ) -> Result<(), &'static str> {
+    fn send_to_client(&self, _client_id: ClientId, _msg: ServerToClientMsg) -> Result<()> {
         unimplemented!()
     }
     fn new_client(
         &mut self,
         _client_id: ClientId,
         _stream: LocalSocketStream,
-    ) -> IpcReceiverWithContext<ClientToServerMsg> {
+    ) -> Result<IpcReceiverWithContext<ClientToServerMsg>> {
         unimplemented!()
     }
-    fn remove_client(&mut self, _client_id: ClientId) {
+    fn remove_client(&mut self, _client_id: ClientId) -> Result<()> {
         unimplemented!()
     }
     fn load_palette(&self) -> Palette {
@@ -96,22 +98,23 @@ impl ServerOsApi for FakeInputOutput {
     fn get_cwd(&self, _pid: Pid) -> Option<PathBuf> {
         unimplemented!()
     }
-    fn write_to_file(&mut self, buf: String, name: Option<String>) {
+    fn write_to_file(&mut self, buf: String, name: Option<String>) -> Result<()> {
         let f: String = match name {
             Some(x) => x,
             None => "tmp-name".to_owned(),
         };
-        self.file_dumps.lock().unwrap().insert(f, buf);
+        self.file_dumps.lock().to_anyhow()?.insert(f, buf);
+        Ok(())
     }
     fn re_run_command_in_terminal(
         &self,
         _terminal_id: u32,
         _run_command: RunCommand,
         _quit_cb: Box<dyn Fn(PaneId, Option<i32>, RunCommand) + Send>, // u32 is the exit status
-    ) -> Result<(RawFd, RawFd), SpawnTerminalError> {
+    ) -> Result<(RawFd, RawFd)> {
         unimplemented!()
     }
-    fn clear_terminal_id(&self, _terminal_id: u32) {
+    fn clear_terminal_id(&self, _terminal_id: u32) -> Result<()> {
         unimplemented!()
     }
 }
@@ -185,9 +188,7 @@ fn create_new_tab(size: Size, default_mode: ModeInfo) -> Tab {
     let index = 0;
     let position = 0;
     let name = String::new();
-    let os_api = Box::new(FakeInputOutput {
-        file_dumps: Arc::new(Mutex::new(HashMap::new())),
-    });
+    let os_api = Box::new(FakeInputOutput::default());
     let senders = ThreadSenders::default().silently_fail_on_send();
     let max_panes = None;
     let mode_info = default_mode;
@@ -223,7 +224,57 @@ fn create_new_tab(size: Size, default_mode: ModeInfo) -> Tab {
         terminal_emulator_colors,
         terminal_emulator_color_codes,
     );
-    tab.apply_layout(PaneLayout::default(), vec![1], index, client_id)
+    tab.apply_layout(PaneLayout::default(), vec![(1, None)], index, client_id)
+        .unwrap();
+    tab
+}
+
+fn create_new_tab_with_os_api(
+    size: Size,
+    default_mode: ModeInfo,
+    os_api: &Box<FakeInputOutput>,
+) -> Tab {
+    set_session_name("test".into());
+    let index = 0;
+    let position = 0;
+    let name = String::new();
+    let os_api = os_api.clone();
+    let senders = ThreadSenders::default().silently_fail_on_send();
+    let max_panes = None;
+    let mode_info = default_mode;
+    let style = Style::default();
+    let draw_pane_frames = true;
+    let client_id = 1;
+    let session_is_mirrored = true;
+    let mut connected_clients = HashSet::new();
+    connected_clients.insert(client_id);
+    let connected_clients = Rc::new(RefCell::new(connected_clients));
+    let character_cell_info = Rc::new(RefCell::new(None));
+    let terminal_emulator_colors = Rc::new(RefCell::new(Palette::default()));
+    let copy_options = CopyOptions::default();
+    let terminal_emulator_color_codes = Rc::new(RefCell::new(HashMap::new()));
+    let sixel_image_store = Rc::new(RefCell::new(SixelImageStore::default()));
+    let mut tab = Tab::new(
+        index,
+        position,
+        name,
+        size,
+        character_cell_info,
+        sixel_image_store,
+        os_api,
+        senders,
+        max_panes,
+        style,
+        mode_info,
+        draw_pane_frames,
+        connected_clients,
+        session_is_mirrored,
+        client_id,
+        copy_options,
+        terminal_emulator_colors,
+        terminal_emulator_color_codes,
+    );
+    tab.apply_layout(PaneLayout::default(), vec![(1, None)], index, client_id)
         .unwrap();
     tab
 }
@@ -233,9 +284,7 @@ fn create_new_tab_with_layout(size: Size, default_mode: ModeInfo, layout: &str)
     let index = 0;
     let position = 0;
     let name = String::new();
-    let os_api = Box::new(FakeInputOutput {
-        file_dumps: Arc::new(Mutex::new(HashMap::new())),
-    });
+    let os_api = Box::new(FakeInputOutput::default());
     let senders = ThreadSenders::default().silently_fail_on_send();
     let max_panes = None;
     let mode_info = default_mode;
@@ -277,7 +326,7 @@ fn create_new_tab_with_layout(size: Size, default_mode: ModeInfo, layout: &str)
         .extract_run_instructions()
         .iter()
         .enumerate()
-        .map(|(i, _)| i as u32)
+        .map(|(i, _)| (i as u32, None))
         .collect();
     tab.apply_layout(tab_layout, pane_ids, index, client_id)
         .unwrap();
@@ -293,9 +342,7 @@ fn create_new_tab_with_mock_pty_writer(
     let index = 0;
     let position = 0;
     let name = String::new();
-    let os_api = Box::new(FakeInputOutput {
-        file_dumps: Arc::new(Mutex::new(HashMap::new())),
-    });
+    let os_api = Box::new(FakeInputOutput::default());
     let mut senders = ThreadSenders::default().silently_fail_on_send();
     senders.replace_to_pty_writer(mock_pty_writer);
     let max_panes = None;
@@ -332,14 +379,8 @@ fn create_new_tab_with_mock_pty_writer(
         terminal_emulator_colors,
         terminal_emulator_color_codes,
     );
-    tab.apply_layout(
-        // LayoutTemplate::default().try_into().unwrap(),
-        PaneLayout::default(),
-        vec![1],
-        index,
-        client_id,
-    )
-    .unwrap();
+    tab.apply_layout(PaneLayout::default(), vec![(1, None)], index, client_id)
+        .unwrap();
     tab
 }
 
@@ -353,9 +394,7 @@ fn create_new_tab_with_sixel_support(
     let index = 0;
     let position = 0;
     let name = String::new();
-    let os_api = Box::new(FakeInputOutput {
-        file_dumps: Arc::new(Mutex::new(HashMap::new())),
-    });
+    let os_api = Box::new(FakeInputOutput::default());
     let senders = ThreadSenders::default().silently_fail_on_send();
     let max_panes = None;
     let mode_info = ModeInfo::default();
@@ -393,7 +432,7 @@ fn create_new_tab_with_sixel_support(
         terminal_emulator_colors,
         terminal_emulator_color_codes,
     );
-    tab.apply_layout(PaneLayout::default(), vec![1], index, client_id)
+    tab.apply_layout(PaneLayout::default(), vec![(1, None)], index, client_id)
         .unwrap();
     tab
 }
@@ -500,6 +539,7 @@ fn dump_screen() {
     let map = Arc::new(Mutex::new(HashMap::new()));
     tab.os_api = Box::new(FakeInputOutput {
         file_dumps: map.clone(),
+        ..Default::default()
     });
     let new_pane_id = PaneId::Terminal(2);
     tab.new_pane(new_pane_id, None, None, Some(client_id))
@@ -507,7 +547,8 @@ fn dump_screen() {
     tab.handle_pty_bytes(2, Vec::from("scratch".as_bytes()))
         .unwrap();
     let file = "/tmp/log.sh";
-    tab.dump_active_terminal_screen(Some(file.to_string()), client_id, false);
+    tab.dump_active_terminal_screen(Some(file.to_string()), client_id, false)
+        .unwrap();
     assert_eq!(
         map.lock().unwrap().get(file).unwrap(),
         "scratch",
@@ -535,7 +576,7 @@ fn new_floating_pane() {
     .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -567,7 +608,7 @@ fn floating_panes_persist_across_toggles() {
     tab.toggle_floating_panes(client_id, None).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -596,7 +637,7 @@ fn toggle_floating_panes_off() {
     tab.toggle_floating_panes(client_id, None).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -626,7 +667,7 @@ fn toggle_floating_panes_on() {
     tab.toggle_floating_panes(client_id, None).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -674,7 +715,7 @@ fn five_new_floating_panes() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -703,7 +744,7 @@ fn increase_floating_pane_size() {
     tab.resize_increase(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -732,7 +773,7 @@ fn decrease_floating_pane_size() {
     tab.resize_decrease(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -761,7 +802,7 @@ fn resize_floating_pane_left() {
     tab.resize_left(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -790,7 +831,7 @@ fn resize_floating_pane_right() {
     tab.resize_right(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -819,7 +860,7 @@ fn resize_floating_pane_up() {
     tab.resize_up(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -848,7 +889,7 @@ fn resize_floating_pane_down() {
     tab.resize_down(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -897,7 +938,7 @@ fn move_floating_pane_focus_left() {
     tab.move_focus_left(client_id);
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -953,7 +994,7 @@ fn move_floating_pane_focus_right() {
     tab.move_focus_right(client_id);
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1008,7 +1049,7 @@ fn move_floating_pane_focus_up() {
     tab.move_focus_up(client_id);
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1064,7 +1105,7 @@ fn move_floating_pane_focus_down() {
     tab.move_focus_down(client_id);
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1122,7 +1163,7 @@ fn move_floating_pane_focus_with_mouse() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1180,7 +1221,7 @@ fn move_pane_focus_with_mouse_to_non_floating_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1238,7 +1279,7 @@ fn drag_pane_with_mouse() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1304,7 +1345,7 @@ fn mark_text_inside_floating_pane() {
     );
     tab.render(&mut output, None).unwrap();
     let (snapshot, cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1362,7 +1403,7 @@ fn resize_tab_with_floating_panes() {
     });
     tab.render(&mut output, None).unwrap();
     let (snapshot, _cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1412,7 +1453,7 @@ fn shrink_whole_tab_with_floating_panes_horizontally_and_vertically() {
     tab.resize_whole_tab(Size { cols: 50, rows: 10 });
     tab.render(&mut output, None).unwrap();
     let (snapshot, _cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1466,7 +1507,7 @@ fn shrink_whole_tab_with_floating_panes_horizontally_and_vertically_and_expand_b
     });
     tab.render(&mut output, None).unwrap();
     let (snapshot, _cursor_coordinates) = take_snapshot_and_cursor_position(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1496,7 +1537,7 @@ fn embed_floating_pane() {
     tab.toggle_pane_embed_or_floating(client_id).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1524,7 +1565,66 @@ fn float_embedded_pane() {
     tab.toggle_pane_embed_or_floating(client_id).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!(snapshot);
+}
+
+#[test]
+fn embed_floating_pane_without_pane_frames() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let mut tab = create_new_tab(size, ModeInfo::default());
+    let new_pane_id = PaneId::Terminal(2);
+    let mut output = Output::default();
+    tab.set_pane_frames(false);
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    tab.new_pane(new_pane_id, None, None, Some(client_id))
+        .unwrap();
+    tab.handle_pty_bytes(
+        2,
+        Vec::from("\n\n\n                   I am scratch terminal".as_bytes()),
+    )
+    .unwrap();
+    tab.toggle_pane_embed_or_floating(client_id).unwrap();
+    tab.render(&mut output, None).unwrap();
+    let snapshot = take_snapshot(
+        output.serialize().unwrap().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!(snapshot);
+}
+
+#[test]
+fn float_embedded_pane_without_pane_frames() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let mut tab = create_new_tab(size, ModeInfo::default());
+    let new_pane_id = PaneId::Terminal(2);
+    let mut output = Output::default();
+    tab.set_pane_frames(false);
+    tab.new_pane(new_pane_id, None, None, Some(client_id))
+        .unwrap();
+    tab.handle_pty_bytes(
+        2,
+        Vec::from("\n\n\n                   I am an embedded pane".as_bytes()),
+    )
+    .unwrap();
+    tab.toggle_pane_embed_or_floating(client_id).unwrap();
+    tab.render(&mut output, None).unwrap();
+    let snapshot = take_snapshot(
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1549,7 +1649,7 @@ fn cannot_float_only_embedded_pane() {
     tab.toggle_pane_embed_or_floating(client_id).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1573,7 +1673,7 @@ fn replacing_existing_wide_characters() {
     tab.handle_pty_bytes(1, pane_content).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1599,7 +1699,7 @@ fn rename_embedded_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1629,7 +1729,7 @@ fn rename_floating_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1651,7 +1751,7 @@ fn wide_characters_in_left_title_side() {
     tab.handle_pty_bytes(1, pane_content).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1678,7 +1778,7 @@ fn save_cursor_position_across_resizes() {
 
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1714,7 +1814,7 @@ fn move_floating_pane_with_sixel_image() {
 
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot_with_sixel(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1752,7 +1852,7 @@ fn floating_pane_above_sixel_image() {
 
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot_with_sixel(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1777,7 +1877,7 @@ fn suppress_tiled_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1805,7 +1905,7 @@ fn suppress_floating_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1831,7 +1931,7 @@ fn close_suppressing_tiled_pane() {
     tab.close_pane(new_pane_id, false);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1862,7 +1962,7 @@ fn close_suppressing_floating_pane() {
     tab.close_pane(editor_pane_id, false);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1889,7 +1989,7 @@ fn suppress_tiled_pane_float_it_and_close() {
     tab.close_pane(new_pane_id, false);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1921,7 +2021,7 @@ fn suppress_floating_pane_embed_it_and_close_it() {
     tab.close_pane(editor_pane_id, false);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1948,7 +2048,7 @@ fn resize_whole_tab_while_tiled_pane_is_suppressed() {
     });
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -1980,7 +2080,7 @@ fn resize_whole_tab_while_floting_pane_is_suppressed() {
     });
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2005,7 +2105,7 @@ fn enter_search_pane() {
     tab.handle_pty_bytes(1, pane_content).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2019,7 +2119,7 @@ fn enter_search_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2032,7 +2132,7 @@ fn enter_search_pane() {
     tab.toggle_search_case_sensitivity(client_id);
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2046,7 +2146,7 @@ fn enter_search_pane() {
 
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2076,7 +2176,7 @@ fn enter_search_floating_pane() {
     tab.handle_pty_bytes(2, pane_content).unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2088,7 +2188,7 @@ fn enter_search_floating_pane() {
         .unwrap();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2366,7 +2466,7 @@ fn tab_with_basic_layout() {
     let mut output = Output::default();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2406,7 +2506,7 @@ fn tab_with_nested_layout() {
     let mut output = Output::default();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2440,7 +2540,7 @@ fn tab_with_nested_uneven_layout() {
     let mut output = Output::default();
     tab.render(&mut output, None).unwrap();
     let snapshot = take_snapshot(
-        output.serialize().get(&client_id).unwrap(),
+        output.serialize().unwrap().get(&client_id).unwrap(),
         size.rows,
         size.cols,
         Palette::default(),
@@ -2531,3 +2631,165 @@ fn pane_faux_scrolling_in_alternate_mode() {
 
     assert_eq!(pty_instruction_bus.clone_output(), expected);
 }
+
+#[test]
+fn move_pane_focus_sends_tty_csi_event() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let tty_stdin_bytes = Arc::new(Mutex::new(BTreeMap::new()));
+    let os_api = Box::new(FakeInputOutput {
+        tty_stdin_bytes: tty_stdin_bytes.clone(),
+        ..Default::default()
+    });
+    let mut tab = create_new_tab_with_os_api(size, ModeInfo::default(), &os_api);
+    let new_pane_id_1 = PaneId::Terminal(2);
+    tab.new_pane(new_pane_id_1, None, None, Some(client_id))
+        .unwrap();
+    tab.handle_pty_bytes(
+        1,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        2,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.move_focus_left(client_id);
+    assert_snapshot!(format!("{:?}", *tty_stdin_bytes.lock().unwrap()));
+}
+
+#[test]
+fn move_floating_pane_focus_sends_tty_csi_event() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let tty_stdin_bytes = Arc::new(Mutex::new(BTreeMap::new()));
+    let os_api = Box::new(FakeInputOutput {
+        tty_stdin_bytes: tty_stdin_bytes.clone(),
+        ..Default::default()
+    });
+    let mut tab = create_new_tab_with_os_api(size, ModeInfo::default(), &os_api);
+    let new_pane_id_1 = PaneId::Terminal(2);
+    let new_pane_id_2 = PaneId::Terminal(3);
+
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    tab.new_pane(new_pane_id_1, None, None, Some(client_id))
+        .unwrap();
+    tab.new_pane(new_pane_id_2, None, None, Some(client_id))
+        .unwrap();
+    tab.handle_pty_bytes(
+        1,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        2,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        3,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.move_focus_left(client_id);
+    assert_snapshot!(format!("{:?}", *tty_stdin_bytes.lock().unwrap()));
+}
+
+#[test]
+fn toggle_floating_panes_on_sends_tty_csi_event() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let tty_stdin_bytes = Arc::new(Mutex::new(BTreeMap::new()));
+    let os_api = Box::new(FakeInputOutput {
+        tty_stdin_bytes: tty_stdin_bytes.clone(),
+        ..Default::default()
+    });
+    let mut tab = create_new_tab_with_os_api(size, ModeInfo::default(), &os_api);
+    let new_pane_id_1 = PaneId::Terminal(2);
+    let new_pane_id_2 = PaneId::Terminal(3);
+
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    tab.new_pane(new_pane_id_1, None, None, Some(client_id))
+        .unwrap();
+    tab.new_pane(new_pane_id_2, None, None, Some(client_id))
+        .unwrap();
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    tab.handle_pty_bytes(
+        1,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        2,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        3,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    assert_snapshot!(format!("{:?}", *tty_stdin_bytes.lock().unwrap()));
+}
+
+#[test]
+fn toggle_floating_panes_off_sends_tty_csi_event() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let client_id = 1;
+    let tty_stdin_bytes = Arc::new(Mutex::new(BTreeMap::new()));
+    let os_api = Box::new(FakeInputOutput {
+        tty_stdin_bytes: tty_stdin_bytes.clone(),
+        ..Default::default()
+    });
+    let mut tab = create_new_tab_with_os_api(size, ModeInfo::default(), &os_api);
+    let new_pane_id_1 = PaneId::Terminal(2);
+    let new_pane_id_2 = PaneId::Terminal(3);
+
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    tab.new_pane(new_pane_id_1, None, None, Some(client_id))
+        .unwrap();
+    tab.new_pane(new_pane_id_2, None, None, Some(client_id))
+        .unwrap();
+    tab.handle_pty_bytes(
+        1,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        2,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.handle_pty_bytes(
+        3,
+        // subscribe to focus events
+        Vec::from("\u{1b}[?1004h".as_bytes()),
+    )
+    .unwrap();
+    tab.toggle_floating_panes(client_id, None).unwrap();
+    assert_snapshot!(format!("{:?}", *tty_stdin_bytes.lock().unwrap()));
+}
