@@ -3,6 +3,7 @@ use crate::panes::{AnsiCode, CharacterStyles, TerminalCharacter, EMPTY_TERMINAL_
 use crate::ui::boundaries::boundary_type;
 use crate::ClientId;
 use zellij_utils::data::{client_id_to_colors, PaletteColor, Style};
+use zellij_utils::errors::prelude::*;
 use zellij_utils::pane_size::Viewport;
 
 use unicode_width::{UnicodeWidthChar, UnicodeWidthStr};
@@ -88,6 +89,7 @@ pub struct PaneFrame {
     pub other_cursors_exist_in_session: bool,
     pub other_focused_clients: Vec<ClientId>,
     exit_status: Option<ExitStatus>,
+    is_first_run: bool,
 }
 
 impl PaneFrame {
@@ -108,6 +110,7 @@ impl PaneFrame {
             other_focused_clients: frame_params.other_focused_clients,
             other_cursors_exist_in_session: frame_params.other_cursors_exist_in_session,
             exit_status: None,
+            is_first_run: false,
         }
     }
     pub fn add_exit_status(&mut self, exit_status: Option<i32>) {
@@ -116,6 +119,9 @@ impl PaneFrame {
             None => Some(ExitStatus::Exited),
         };
     }
+    pub fn indicate_first_run(&mut self) {
+        self.is_first_run = true;
+    }
     fn client_cursor(&self, client_id: ClientId) -> Vec<TerminalCharacter> {
         let color = client_id_to_colors(client_id, self.style.colors);
         background_color(" ", color.map(|c| c.0))
@@ -600,24 +606,22 @@ impl PaneFrame {
             _ => self.empty_title_line(),
         }
     }
-    fn render_title(&self) -> Vec<TerminalCharacter> {
+    fn render_title(&self) -> Result<Vec<TerminalCharacter>> {
         let total_title_length = self.geom.cols.saturating_sub(2); // 2 for the left and right corners
 
         self.render_title_middle(total_title_length)
             .map(|(middle, middle_length)| self.title_line_with_middle(middle, &middle_length))
             .or_else(|| Some(self.title_line_without_middle()))
-            .unwrap()
+            .with_context(|| format!("failed to render title '{}'", self.title))
     }
-    fn render_held_undertitle(&self) -> Vec<TerminalCharacter> {
+    fn render_held_undertitle(&self) -> Result<Vec<TerminalCharacter>> {
         let max_undertitle_length = self.geom.cols.saturating_sub(2); // 2 for the left and right corners
-        let exit_status = self.exit_status.unwrap(); // unwrap is safe because we only call this if
-
-        let (mut first_part, first_part_len) = self.first_held_title_part_full(exit_status);
+        let (mut first_part, first_part_len) = self.first_exited_held_title_part_full();
         let mut left_boundary =
             foreground_color(self.get_corner(boundary_type::BOTTOM_LEFT), self.color);
         let mut right_boundary =
             foreground_color(self.get_corner(boundary_type::BOTTOM_RIGHT), self.color);
-        if self.is_main_client {
+        let res = if self.is_main_client {
             let (mut second_part, second_part_len) = self.second_held_title_part_full();
             let full_text_len = first_part_len + second_part_len;
             if full_text_len <= max_undertitle_length {
@@ -665,23 +669,29 @@ impl PaneFrame {
             } else {
                 self.empty_undertitle(max_undertitle_length)
             }
-        }
+        };
+        Ok(res)
     }
-    pub fn render(&self) -> (Vec<CharacterChunk>, Option<String>) {
+    pub fn render(&self) -> Result<(Vec<CharacterChunk>, Option<String>)> {
+        let err_context = || "failed to render pane frame";
         let mut character_chunks = vec![];
         for row in 0..self.geom.rows {
             if row == 0 {
                 // top row
-                let title = self.render_title();
+                let title = self.render_title().with_context(err_context)?;
                 let x = self.geom.x;
                 let y = self.geom.y + row;
                 character_chunks.push(CharacterChunk::new(title, x, y));
             } else if row == self.geom.rows - 1 {
                 // bottom row
-                if self.exit_status.is_some() {
+                if self.exit_status.is_some() || self.is_first_run {
                     let x = self.geom.x;
                     let y = self.geom.y + row;
-                    character_chunks.push(CharacterChunk::new(self.render_held_undertitle(), x, y));
+                    character_chunks.push(CharacterChunk::new(
+                        self.render_held_undertitle().with_context(err_context)?,
+                        x,
+                        y,
+                    ));
                 } else {
                     let mut bottom_row = vec![];
                     for col in 0..self.geom.cols {
@@ -716,15 +726,12 @@ impl PaneFrame {
                 character_chunks.push(CharacterChunk::new(boundary_character_right, x, y));
             }
         }
-        (character_chunks, None)
+        Ok((character_chunks, None))
     }
-    fn first_held_title_part_full(
-        &self,
-        exit_status: ExitStatus,
-    ) -> (Vec<TerminalCharacter>, usize) {
+    fn first_exited_held_title_part_full(&self) -> (Vec<TerminalCharacter>, usize) {
         // (title part, length)
-        match exit_status {
-            ExitStatus::Code(exit_code) => {
+        match self.exit_status {
+            Some(ExitStatus::Code(exit_code)) => {
                 let mut first_part = vec![];
                 let left_bracket = " [ ";
                 let exited_text = "EXIT CODE: ";
@@ -750,7 +757,7 @@ impl PaneFrame {
                         + right_bracket.len(),
                 )
             },
-            ExitStatus::Exited => {
+            Some(ExitStatus::Exited) => {
                 let mut first_part = vec![];
                 let left_bracket = " [ ";
                 let exited_text = "EXITED";
@@ -766,15 +773,20 @@ impl PaneFrame {
                     left_bracket.len() + exited_text.len() + right_bracket.len(),
                 )
             },
+            None => (foreground_color(boundary_type::HORIZONTAL, self.color), 1),
         }
     }
     fn second_held_title_part_full(&self) -> (Vec<TerminalCharacter>, usize) {
         // (title part, length)
         let mut second_part = vec![];
-        let left_enter_bracket = "<";
+        let left_enter_bracket = if self.is_first_run { " <" } else { "<" };
         let enter_text = "ENTER";
         let right_enter_bracket = ">";
-        let enter_tip = " to re-run, ";
+        let enter_tip = if self.is_first_run {
+            " to run, "
+        } else {
+            " to re-run, "
+        };
         let left_break_bracket = "<";
         let break_text = "Ctrl-c";
         let right_break_bracket = ">";
