@@ -2,7 +2,7 @@ use super::{screen_thread_main, CopyOptions, Screen, ScreenInstruction};
 use crate::panes::PaneId;
 use crate::{
     channels::SenderWithContext,
-    os_input_output::{AsyncReader, Pid, ServerOsApi, SpawnTerminalError},
+    os_input_output::{AsyncReader, Pid, ServerOsApi},
     route::route_action,
     thread_bus::Bus,
     ClientId, ServerInstruction, SessionMetaData, ThreadSenders,
@@ -10,7 +10,7 @@ use crate::{
 use insta::assert_snapshot;
 use std::path::PathBuf;
 use zellij_utils::cli::CliAction;
-use zellij_utils::errors::ErrorContext;
+use zellij_utils::errors::{prelude::*, ErrorContext};
 use zellij_utils::input::actions::{Action, Direction, ResizeDirection};
 use zellij_utils::input::command::{RunCommand, TerminalAction};
 use zellij_utils::input::layout::{PaneLayout, SplitDirection};
@@ -25,7 +25,7 @@ use std::sync::{Arc, Mutex};
 
 use crate::{pty::PtyInstruction, wasm_vm::PluginInstruction};
 use zellij_utils::ipc::PixelDimensions;
-use zellij_utils::nix;
+
 use zellij_utils::{
     channels::{self, ChannelWithContext, Receiver},
     data::{InputMode, ModeInfo, Palette, PluginCapabilities},
@@ -127,43 +127,45 @@ struct FakeInputOutput {
 }
 
 impl ServerOsApi for FakeInputOutput {
-    fn set_terminal_size_using_terminal_id(&self, _terminal_id: u32, _cols: u16, _rows: u16) {
+    fn set_terminal_size_using_terminal_id(
+        &self,
+        _terminal_id: u32,
+        _cols: u16,
+        _rows: u16,
+    ) -> Result<()> {
         // noop
+        Ok(())
     }
     fn spawn_terminal(
         &self,
         _file_to_open: TerminalAction,
         _quit_db: Box<dyn Fn(PaneId, Option<i32>, RunCommand) + Send>,
         _default_editor: Option<PathBuf>,
-    ) -> Result<(u32, RawFd, RawFd), SpawnTerminalError> {
+    ) -> Result<(u32, RawFd, RawFd)> {
         unimplemented!()
     }
-    fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize, nix::Error> {
+    fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize> {
         unimplemented!()
     }
     fn async_file_reader(&self, _fd: RawFd) -> Box<dyn AsyncReader> {
         unimplemented!()
     }
-    fn write_to_tty_stdin(&self, _id: u32, _buf: &[u8]) -> Result<usize, nix::Error> {
+    fn write_to_tty_stdin(&self, _id: u32, _buf: &[u8]) -> Result<usize> {
         unimplemented!()
     }
-    fn tcdrain(&self, _id: u32) -> Result<(), nix::Error> {
+    fn tcdrain(&self, _id: u32) -> Result<()> {
         unimplemented!()
     }
-    fn kill(&self, _pid: Pid) -> Result<(), nix::Error> {
+    fn kill(&self, _pid: Pid) -> Result<()> {
         unimplemented!()
     }
-    fn force_kill(&self, _pid: Pid) -> Result<(), nix::Error> {
+    fn force_kill(&self, _pid: Pid) -> Result<()> {
         unimplemented!()
     }
     fn box_clone(&self) -> Box<dyn ServerOsApi> {
         Box::new((*self).clone())
     }
-    fn send_to_client(
-        &self,
-        client_id: ClientId,
-        msg: ServerToClientMsg,
-    ) -> Result<(), &'static str> {
+    fn send_to_client(&self, client_id: ClientId, msg: ServerToClientMsg) -> Result<()> {
         self.server_to_client_messages
             .lock()
             .unwrap()
@@ -176,10 +178,10 @@ impl ServerOsApi for FakeInputOutput {
         &mut self,
         _client_id: ClientId,
         _stream: LocalSocketStream,
-    ) -> IpcReceiverWithContext<ClientToServerMsg> {
+    ) -> Result<IpcReceiverWithContext<ClientToServerMsg>> {
         unimplemented!()
     }
-    fn remove_client(&mut self, _client_id: ClientId) {
+    fn remove_client(&mut self, _client_id: ClientId) -> Result<()> {
         unimplemented!()
     }
     fn load_palette(&self) -> Palette {
@@ -188,23 +190,24 @@ impl ServerOsApi for FakeInputOutput {
     fn get_cwd(&self, _pid: Pid) -> Option<PathBuf> {
         unimplemented!()
     }
-    fn write_to_file(&mut self, contents: String, filename: Option<String>) {
+    fn write_to_file(&mut self, contents: String, filename: Option<String>) -> Result<()> {
         if let Some(filename) = filename {
             self.fake_filesystem
                 .lock()
                 .unwrap()
                 .insert(filename, contents);
         }
+        Ok(())
     }
     fn re_run_command_in_terminal(
         &self,
         _terminal_id: u32,
         _run_command: RunCommand,
         _quit_cb: Box<dyn Fn(PaneId, Option<i32>, RunCommand) + Send>, // u32 is the exit status
-    ) -> Result<(RawFd, RawFd), SpawnTerminalError> {
+    ) -> Result<(RawFd, RawFd)> {
         unimplemented!()
     }
-    fn clear_terminal_id(&self, _terminal_id: u32) {
+    fn clear_terminal_id(&self, _terminal_id: u32) -> Result<()> {
         unimplemented!()
     }
 }
@@ -284,7 +287,7 @@ impl MockScreen {
         let pane_count = pane_layout.extract_run_instructions().len();
         let mut pane_ids = vec![];
         for i in 0..pane_count {
-            pane_ids.push(i as u32);
+            pane_ids.push((i as u32, None));
         }
         let _ = self.to_screen.send(ScreenInstruction::NewTab(
             pane_layout,
@@ -297,7 +300,7 @@ impl MockScreen {
         let pane_count = tab_layout.extract_run_instructions().len();
         let mut pane_ids = vec![];
         for i in 0..pane_count {
-            pane_ids.push(i as u32);
+            pane_ids.push((i as u32, None));
         }
         let _ = self.to_screen.send(ScreenInstruction::NewTab(
             tab_layout,
@@ -427,7 +430,7 @@ macro_rules! log_actions_in_thread {
 fn new_tab(screen: &mut Screen, pid: u32) {
     let client_id = 1;
     screen
-        .new_tab(PaneLayout::default(), vec![pid], client_id)
+        .new_tab(PaneLayout::default(), vec![(pid, None)], client_id)
         .expect("TEST");
 }
 
@@ -1821,6 +1824,8 @@ pub fn send_cli_new_pane_action_with_default_parameters() {
         cwd: None,
         floating: false,
         name: None,
+        close_on_exit: false,
+        start_suspended: false,
     };
     send_cli_action_to_server(
         &session_metadata,
@@ -1859,6 +1864,8 @@ pub fn send_cli_new_pane_action_with_split_direction() {
         cwd: None,
         floating: false,
         name: None,
+        close_on_exit: false,
+        start_suspended: false,
     };
     send_cli_action_to_server(
         &session_metadata,
@@ -1897,6 +1904,8 @@ pub fn send_cli_new_pane_action_with_command_and_cwd() {
         cwd: Some("/some/folder".into()),
         floating: false,
         name: None,
+        close_on_exit: false,
+        start_suspended: false,
     };
     send_cli_action_to_server(
         &session_metadata,
@@ -1934,6 +1943,7 @@ pub fn send_cli_edit_action_with_default_parameters() {
         direction: None,
         line_number: None,
         floating: false,
+        cwd: None,
     };
     send_cli_action_to_server(
         &session_metadata,
@@ -1971,6 +1981,7 @@ pub fn send_cli_edit_action_with_line_number() {
         direction: None,
         line_number: Some(100),
         floating: false,
+        cwd: None,
     };
     send_cli_action_to_server(
         &session_metadata,
@@ -2008,6 +2019,7 @@ pub fn send_cli_edit_action_with_split_direction() {
         direction: Some(Direction::Down),
         line_number: None,
         floating: false,
+        cwd: None,
     };
     send_cli_action_to_server(
         &session_metadata,
