@@ -149,7 +149,9 @@ pub(crate) fn wasm_thread_main(
     let plugin_global_data_dir = plugin_dir.join("data");
 
     #[cfg(not(feature = "disable_automatic_asset_installation"))]
-    fs::create_dir_all(&plugin_global_data_dir).unwrap_or_else(|e| log::error!("{:?}", e));
+    fs::create_dir_all(&plugin_global_data_dir)
+        .context("failed to create plugin asset directory")
+        .non_fatal();
 
     loop {
         let (event, mut err_ctx) = bus.recv().expect("failed to receive event on channel");
@@ -351,7 +353,6 @@ fn start_plugin(
     // The plugins blob as stored on the filesystem
     let wasm_bytes = plugin
         .resolve_wasm_bytes(&data_dir.join("plugins/"))
-        .context("cannot resolve wasm bytes")
         .with_context(err_context)
         .fatal();
 
