@@ -48,15 +48,6 @@ pub struct CliArgs {
     pub debug: bool,
 }
 
-impl CliArgs {
-    pub fn should_clean_config(&self) -> bool {
-        match &self.command {
-            Some(Command::Setup(ref setup)) => setup.clean,
-            _ => false,
-        }
-    }
-}
-
 #[derive(Debug, Subcommand, Clone, Serialize, Deserialize)]
 pub enum Command {
     /// Change the behaviour of zellij
@@ -146,6 +137,14 @@ pub enum Sessions {
         /// Name of the new pane
         #[clap(short, long, value_parser)]
         name: Option<String>,
+
+        /// Close the pane immediately when its command exits
+        #[clap(short, long, value_parser, default_value("false"), takes_value(false))]
+        close_on_exit: bool,
+
+        /// Start the command suspended, only running after you first presses ENTER
+        #[clap(short, long, value_parser, default_value("false"), takes_value(false))]
+        start_suspended: bool,
     },
     /// Edit file with default $EDITOR / $VISUAL
     #[clap(visible_alias = "e")]
@@ -163,6 +162,10 @@ pub enum Sessions {
         /// Open the new pane in floating mode
         #[clap(short, long, value_parser, default_value("false"), takes_value(false))]
         floating: bool,
+
+        /// Change the working directory of the editor
+        #[clap(long, value_parser)]
+        cwd: Option<PathBuf>,
     },
     ConvertConfig {
         old_config_file: PathBuf,
@@ -246,6 +249,27 @@ pub enum CliAction {
         /// Name of the new pane
         #[clap(short, long, value_parser)]
         name: Option<String>,
+
+        /// Close the pane immediately when its command exits
+        #[clap(
+            short,
+            long,
+            value_parser,
+            default_value("false"),
+            takes_value(false),
+            requires("command")
+        )]
+        close_on_exit: bool,
+        /// Start the command suspended, only running it after the you first press ENTER
+        #[clap(
+            short,
+            long,
+            value_parser,
+            default_value("false"),
+            takes_value(false),
+            requires("command")
+        )]
+        start_suspended: bool,
     },
     /// Open the specified file in a new zellij pane with your default EDITOR
     Edit {
@@ -262,6 +286,10 @@ pub enum CliAction {
         /// Open the new pane in floating mode
         #[clap(short, long, value_parser, default_value("false"), takes_value(false))]
         floating: bool,
+
+        /// Change the working directory of the editor
+        #[clap(long, value_parser)]
+        cwd: Option<PathBuf>,
     },
     /// Switch input mode of all connected clients [locked|pane|tab|resize|move|search|session]
     SwitchMode { input_mode: InputMode },
