@@ -224,6 +224,7 @@ pub enum Action {
     Search(SearchDirection),
     /// Toggle case sensitivity of search
     SearchToggleOption(SearchOption),
+    ToggleMouseMode,
 }
 
 impl Action {
@@ -258,17 +259,22 @@ impl Action {
                 cwd,
                 floating,
                 name,
+                close_on_exit,
+                start_suspended,
             } => {
                 if !command.is_empty() {
                     let mut command = command.clone();
                     let (command, args) = (PathBuf::from(command.remove(0)), command);
                     let cwd = cwd.or_else(|| std::env::current_dir().ok());
+                    let hold_on_start = start_suspended;
+                    let hold_on_close = !close_on_exit;
                     let run_command_action = RunCommandAction {
                         command,
                         args,
                         cwd,
                         direction,
-                        hold_on_close: true,
+                        hold_on_close,
+                        hold_on_start,
                     };
                     if floating {
                         Ok(vec![Action::NewFloatingPane(
@@ -295,10 +301,12 @@ impl Action {
                 file,
                 line_number,
                 floating,
+                cwd,
             } => {
                 let mut file = file;
+                let cwd = cwd.or_else(|| std::env::current_dir().ok());
                 if file.is_relative() {
-                    if let Some(cwd) = std::env::current_dir().ok() {
+                    if let Some(cwd) = cwd {
                         file = cwd.join(file);
                     }
                 }
