@@ -1,7 +1,6 @@
 mod kdl_layout_parser;
 use crate::data::{InputMode, Key, Palette, PaletteColor};
 use crate::envs::EnvironmentVariables;
-use crate::input::command::RunCommand;
 use crate::input::config::{Config, ConfigError, KdlError};
 use crate::input::keybinds::Keybinds;
 use crate::input::layout::{Layout, RunPlugin, RunPluginLocation};
@@ -65,6 +64,7 @@ macro_rules! parse_kdl_action_arguments {
                 "Copy" => Ok(Action::Copy),
                 "Confirm" => Ok(Action::Confirm),
                 "Deny" => Ok(Action::Deny),
+                "ToggleMouseMode" => Ok(Action::ToggleMouseMode),
                 _ => Err(ConfigError::new_kdl_error(
                     format!("Unsupported action: {:?}", $action_name),
                     $action_node.span().offset(),
@@ -338,6 +338,16 @@ pub fn kdl_child_string_value_for_entry<'a>(
         .and_then(|cwd_value| cwd_value.value().as_string())
 }
 
+pub fn kdl_child_bool_value_for_entry<'a>(
+    command_metadata: &'a KdlDocument,
+    entry_name: &'a str,
+) -> Option<bool> {
+    command_metadata
+        .get(entry_name)
+        .and_then(|cwd| cwd.entries().iter().next())
+        .and_then(|cwd_value| cwd_value.value().as_bool())
+}
+
 impl Action {
     pub fn new_from_bytes(
         action_name: &str,
@@ -653,6 +663,9 @@ impl TryFrom<&KdlNode> for Action {
             "UndoRenameTab" => {
                 parse_kdl_action_arguments!(action_name, action_arguments, kdl_action)
             },
+            "ToggleMouseMode" => {
+                parse_kdl_action_arguments!(action_name, action_arguments, kdl_action)
+            },
             "Detach" => parse_kdl_action_arguments!(action_name, action_arguments, kdl_action),
             "Copy" => parse_kdl_action_arguments!(action_name, action_arguments, kdl_action),
             "Confirm" => parse_kdl_action_arguments!(action_name, action_arguments, kdl_action),
@@ -737,12 +750,20 @@ impl TryFrom<&KdlNode> for Action {
                 let direction = command_metadata
                     .and_then(|c_m| kdl_child_string_value_for_entry(c_m, "direction"))
                     .and_then(|direction_string| Direction::from_str(direction_string).ok());
+                let hold_on_close = command_metadata
+                    .and_then(|c_m| kdl_child_bool_value_for_entry(c_m, "close_on_exit"))
+                    .and_then(|close_on_exit| Some(!close_on_exit))
+                    .unwrap_or(true);
+                let hold_on_start = command_metadata
+                    .and_then(|c_m| kdl_child_bool_value_for_entry(c_m, "start_suspended"))
+                    .unwrap_or(false);
                 let run_command_action = RunCommandAction {
                     command: PathBuf::from(command),
                     args,
                     cwd,
                     direction,
-                    hold_on_close: true,
+                    hold_on_close,
+                    hold_on_start,
                 };
                 Ok(Action::Run(run_command_action))
             },
@@ -1460,32 +1481,6 @@ impl Keybinds {
     }
 }
 
-impl RunCommand {
-    pub fn from_kdl(kdl_node: &KdlNode) -> Result<Self, ConfigError> {
-        let command = PathBuf::from(kdl_get_child_entry_string_value!(kdl_node, "cmd").ok_or(
-            ConfigError::new_kdl_error(
-                "Command must have a cmd value".into(),
-                kdl_node.span().offset(),
-                kdl_node.span().len(),
-            ),
-        )?);
-        let cwd = kdl_get_child_entry_string_value!(kdl_node, "cwd").map(|c| PathBuf::from(c));
-        let args = match kdl_get_child!(kdl_node, "args") {
-            Some(kdl_args) => kdl_string_arguments!(kdl_args)
-                .iter()
-                .map(|s| String::from(*s))
-                .collect(),
-            None => vec![],
-        };
-        Ok(RunCommand {
-            command,
-            args,
-            cwd,
-            hold_on_close: true,
-        })
-    }
-}
-
 impl Config {
     pub fn from_kdl(kdl_config: &str, base_config: Option<Config>) -> Result<Config, ConfigError> {
         let mut config = base_config.unwrap_or_else(|| Config::default());
@@ -1587,10 +1582,8 @@ impl Themes {
         let themes = Themes::from_data(themes);
         Ok(themes)
     }
-}
 
-impl Theme {
-    pub fn from_path(path_to_theme_file: PathBuf) -> Result<(String, Self), ConfigError> {
+    pub fn from_path(path_to_theme_file: PathBuf) -> Result<Self, ConfigError> {
         // String is the theme name
         let mut file = File::open(path_to_theme_file.clone())?;
         let mut kdl_config = String::new();
@@ -1602,34 +1595,6 @@ impl Theme {
             kdl_config.span().len(),
         ))?;
         let all_themes_in_file = Themes::from_kdl(kdl_themes)?;
-        let theme_file_name = path_to_theme_file
-            .file_name()
-            .ok_or(ConfigError::new_kdl_error(
-                "Failed to find file name".into(),
-                kdl_config.span().offset(),
-                kdl_config.span().len(),
-            ))?
-            .to_string_lossy()
-            .to_string();
-        if let Some(theme_name) = theme_file_name.strip_suffix(".kdl") {
-            let theme =
-                all_themes_in_file
-                    .get_theme(theme_name)
-                    .ok_or(ConfigError::new_kdl_error(
-                        format!(
-                            "Not theme with name {} found in file {:?}",
-                            theme_name, path_to_theme_file
-                        ),
-                        kdl_config.span().offset(),
-                        kdl_config.span().len(),
-                    ))?;
-            Ok((theme_name.to_string(), theme.clone()))
-        } else {
-            Err(ConfigError::new_kdl_error(
-                "no theme file found".into(),
-                kdl_config.span().offset(),
-                kdl_config.span().len(),
-            ))
-        }
+        Ok(all_themes_in_file)
     }
 }
