@@ -34,6 +34,12 @@ pub struct Size {
     pub cols: usize,
 }
 
+#[derive(Clone, Copy, Default, PartialEq, Eq, Debug, Serialize, Deserialize)]
+pub struct SizeInPixels {
+    pub height: usize,
+    pub width: usize,
+}
+
 #[derive(Eq, Clone, Copy, PartialEq, Debug, Serialize, Deserialize)]
 pub struct Dimension {
     pub constraint: Constraint,
@@ -77,6 +83,29 @@ impl Dimension {
         self.inner = inner;
     }
 
+    pub fn adjust_inner(&mut self, full_size: usize) -> f64 {
+        // returns the leftover from
+        // rounding if any
+        // TODO: elsewhere?
+        match self.constraint {
+            Constraint::Percent(percent) => {
+                let new_inner = (percent / 100.0) * full_size as f64;
+                let rounded = new_inner.floor();
+                let leftover = rounded - new_inner;
+                self.set_inner(rounded as usize);
+                leftover
+                // self.set_inner(((percent / 100.0) * full_size as f64).round() as usize);
+            },
+            Constraint::Fixed(fixed_size) => {
+                self.set_inner(fixed_size);
+                0.0
+            },
+        }
+    }
+    pub fn increase_inner(&mut self, by: usize) {
+        self.inner += by;
+    }
+
     pub fn is_fixed(&self) -> bool {
         matches!(self.constraint, Constraint::Fixed(_))
     }
@@ -101,6 +130,9 @@ impl PaneGeom {
             && self.y <= row
             && row < self.y + self.rows.as_usize()
     }
+    pub fn is_at_least_minimum_size(&self) -> bool {
+        self.rows.as_usize() > 0 && self.cols.as_usize() > 0
+    }
 }
 
 impl Offset {
