@@ -1,4 +1,4 @@
-use crate::input::theme::Theme;
+use crate::input::theme::Themes;
 use crate::{
     cli::{CliArgs, Command},
     consts::{
@@ -211,14 +211,9 @@ impl Setup {
     ///    (`layout.yaml` / `zellij --layout`)
     /// 3. config options (`config.yaml`)
     pub fn from_cli_args(cli_args: &CliArgs) -> Result<(Config, Layout, Options), ConfigError> {
-        let clean = cli_args.should_clean_config();
         // note that this can potentially exit the process
         Setup::handle_setup_commands(cli_args);
-        let config = if clean {
-            Config::default()
-        } else {
-            Config::try_from(cli_args)?
-        };
+        let config = Config::try_from(cli_args)?;
         let cli_config_options: Option<Options> =
             if let Some(Command::Options(options)) = cli_args.command.clone() {
                 Some(options.into())
@@ -241,10 +236,8 @@ impl Setup {
                 for entry in (theme_dir.read_dir()?).flatten() {
                     if let Some(extension) = entry.path().extension() {
                         if extension == "kdl" {
-                            match Theme::from_path(entry.path()) {
-                                Ok((theme_name, theme)) => {
-                                    config.themes.insert(theme_name, theme);
-                                },
+                            match Themes::from_path(entry.path()) {
+                                Ok(themes) => config.themes = config.themes.merge(themes),
                                 Err(e) => {
                                     log::error!("error loading theme file: {:?}", e);
                                 },
